class DocumentParserBase:
    """
    文档解析器基类
    """

    def parse(self, file, configs):
        """
        文档解析
        :param file:文件
        :param configs:提取配置
        :return: 无
        """
        pass
